@@ -50,12 +50,18 @@ class Git(Fetch):
         if 'protocol' in ud.parm:
             ud.proto = ud.parm['protocol']
 
-        tag = data.getVar("SRCREV", d, 0)
-        if 'tag' in ud.parm and len(ud.parm['tag']) == 40:
-            ud.tag = ud.parm['tag']
-        elif tag and "get_srcrev" not in tag and len(tag) == 40:
+        ud.branch = ud.parm.get("branch", "")
+
+        tag = Fetch.srcrev_internal_helper(ud, d)
+        if tag is True:
+            ud.tag = self.latest_revision(url, ud, d)	
+        elif tag:
             ud.tag = tag
-        else:
+
+        if not ud.tag:
+            ud.tag = self.latest_revision(url, ud, d)	
+
+        if ud.tag == "master":
             ud.tag = self.latest_revision(url, ud, d)
 
         ud.localfile = data.expand('git_%s%s_%s.tar.gz' % (ud.host, ud.path.replace('/', '.'), ud.tag), d)
@@ -89,16 +95,16 @@ class Git(Fetch):
         os.chdir(repodir)
         # Remove all but the .git directory
         runfetchcmd("rm * -Rf", d)
-        runfetchcmd("git pull %s://%s%s" % (ud.proto, ud.host, ud.path), d)
+        runfetchcmd("git fetch %s://%s%s" % (ud.proto, ud.host, ud.path), d)
         runfetchcmd("git pull --tags %s://%s%s" % (ud.proto, ud.host, ud.path), d)
         runfetchcmd("git prune-packed", d)
         runfetchcmd("git pack-redundant --all | xargs -r rm", d)
-        # old method of downloading tags
-        #runfetchcmd("rsync -a --verbose --stats --progress rsync://%s%s/ %s" % (ud.host, ud.path, os.path.join(repodir, ".git", "")), d)
 
         os.chdir(repodir)
-        bb.msg.note(1, bb.msg.domain.Fetcher, "Creating tarball of git repository")
-        runfetchcmd("tar -czf %s %s" % (repofile, os.path.join(".", ".git", "*") ), d)
+        mirror_tarballs = data.getVar("BB_GENERATE_MIRROR_TARBALLS", d, True)
+        if mirror_tarballs != "0": 
+            bb.msg.note(1, bb.msg.domain.Fetcher, "Creating tarball of git repository")
+            runfetchcmd("tar -czf %s %s" % (repofile, os.path.join(".", ".git", "*") ), d)
 
         if os.path.exists(codir):
             prunedir(codir)
@@ -125,7 +131,12 @@ class Git(Fetch):
         return "git:" + ud.host + ud.path.replace('/', '.')
 
     def _latest_revision(self, url, ud, d):
-
-        output = runfetchcmd("git ls-remote %s://%s%s" % (ud.proto, ud.host, ud.path), d, True)
+        """
+        Compute the HEAD revision for the url
+        """
+        output = runfetchcmd("git ls-remote %s://%s%s %s" % (ud.proto, ud.host, ud.path, ud.branch), d, True)
         return output.split()[0]
 
+    def _build_revision(self, url, ud, d):
+        return ud.tag
+
