@@ -3,25 +3,27 @@ BitBake Parsers
 
 File parsers for the BitBake build tools.
 
-Copyright (C) 2003, 2004  Chris Larson
-Copyright (C) 2003, 2004  Phil Blundell
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA.
-
-Based on functions from the base bb module, Copyright 2003 Holger Schurig
 """
 
+
+# Copyright (C) 2003, 2004  Chris Larson
+# Copyright (C) 2003, 2004  Phil Blundell
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Based on functions from the base bb module, Copyright 2003 Holger Schurig
+
 __all__ = [ 'ParseError', 'SkipPackage', 'cached_mtime', 'mark_dependency',
             'supports', 'handle', 'init' ]
 handlers = []
@@ -37,11 +39,16 @@ class SkipPackage(Exception):
 __mtime_cache = {}
 def cached_mtime(f):
     if not __mtime_cache.has_key(f):
-        update_mtime(f)
+        __mtime_cache[f] = os.stat(f)[8]
     return __mtime_cache[f]
 
-def update_mtime(f):
-    __mtime_cache[f] = os.stat(f)[8]
+def cached_mtime_noerror(f):
+    if not __mtime_cache.has_key(f):
+        try:
+            __mtime_cache[f] = os.stat(f)[8]
+        except OSError:
+            return 0
+    return __mtime_cache[f]
 
 def mark_dependency(d, f):
     if f.startswith('./'):
